import numpy as np
from sklearn import linear_model
import matplotlib.pyplot as plt

# Los datos hechos en "línea"
n = 12
vx = np.linspace( 0, 10, n )
k = 2.0*np.pi/10.0
vy = np.sin( k*vx )

# Fijamos la semilla para generar la misma secuencia
# de números
np.random.seed( 40 )
vy = vy + 0.5*(np.random.rand( n ) - 0.5)

# De entrada tenemos el conjunto de puntos
# en vx, vy

# Modelo:
#   y = a3 x^3 + a2 x^2 + a1 x + a0

X = np.zeros( (n,3) ) 
X[:,0] = vx * vx * vx    # x*x*x
X[:,1] = vx * vx         # x*x
X[:,2] = vx              # x
print( X )

print( "X:", X.shape )

# Lo
reg = linear_model.LinearRegression( )
reg.fit( X, vy ) 

va = reg.coef_
print( va )
a0 = reg.intercept_
print( a0 )

r2 = reg.score( X, vy ) 
print( "R2:", r2 )

fig, ex = plt.subplots( )

vv = np.linspace( 0, 10 )
V = np.zeros( (50,3) ) 
V[:,0] = vv * vv * vv    # x*x
V[:,1] = vv * vv         # x*x
V[:,2] = vv              # x
#   X    
# n x 2  2 x 1
#    n x 1
# y' = X * va^T + a0
vyp = V @ va.T + a0
ex.plot( vv, vyp )

ex.scatter( vx, vy )

plt.show( )
